package main
 
import (
    "fmt"
    "github.com/hyperledger/fabric/core/chaincode/shim"
    pb "github.com/hyperledger/fabric/protos/peer"
)
 
// 实例化合约
func main() {
    if err := shim.Start(NewCrossChainTest()); err != nil {
        fmt.Printf("Error starting Biz chaincode: %s", err)
    }
}
 
// 跨链合约数据结构
type CrossChainTest struct {
}
 
// 构造跨链合约
func NewCrossChainTest() *CrossChainTest {
    return &CrossChainTest{
    }
}
 
// 初始化 Init 函数
func (bs *CrossChainTest) Init(stub shim.ChaincodeStubInterface) pb.Response {
    return shim.Success([]byte("Init success"))
}
 
/*
 * 合约调用
 */
func (bs *CrossChainTest) Invoke(stub shim.ChaincodeStubInterface) pb.Response {
    fn, args := stub.GetFunctionAndParameters()
 
    switch fn {
 
    // 用户自定义方法
    case "testSendMessage":
        // 对外发送的消息
        // args[0]: 跨链链码名字，从 Baas 平台上查询获取
        // args[1]: 目的区块链域名
        // args[2]: 接收者身份
        //          如果目的区块链是蚂蚁链，账号为账号地址 (32 字节) hex 字符串，不要以 0x 开头
        //          如果目的区块链时 Fabric，账号为接收消息的链码名字进行 sha256 后哈希值的 hex 字符串
        // args[3]: 发送消息内容
        // args[4]: 发送消息内容 nounce
        if len(args) != 5 {
            fmt.Println("Unexpected args len")
            return shim.Error("Unexpected args len")
        }
        fmt.Printf("CrossChainTest send message to %s::%s, content is %s\n", args[0], args[1], args[2])
 
        // 调用跨链链码
        var (
            cc = args[0]   // 跨链utility链码名字
        )
        var args_cross = [][]byte {
            []byte("sendMessage"),  // 发送跨链消息
            []byte(args[1]),  // 目标区块链域名
            []byte(args[2]),  // 接收消息的 mychain 客户合约地址
            []byte(args[3]),  // 发送的消息
            []byte(args[4]),  // 发送的消息 nounce
        }
        re := stub.InvokeChaincode(cc, args_cross, stub.GetChannelID())
        return re
 
    //客户合约实现接收有序消息接口
    case "recvMessage": // 接收消息
        return bs.recvMessage(stub, args[0], args[1], args[2])
 
    default:
        return shim.Error("Method not found")
    }
}
 
//客户合约必须实现接口
func (bs *CrossChainTest) recvMessage(stub shim.ChaincodeStubInterface, sourceDomain string, sourceIdentity string, message string) pb.Response {
    //  sourceDomain stirng,   // 消息来源区块链的域名
    //  sourceIdentity string, // 消息发送者身份
    //  message string)        // 消息内容
    //  补充具体实现
    fmt.Printf("CrossChainTest recv message from domain:%s, identity:%s, msg:%s\n", sourceDomain, sourceIdentity, message)
    return shim.Success(nil)
}
